<?php

App::uses('AppComponent', 'Controller/Component');
App::uses('CakeText', 'Utility');

class BasketComponent extends AppComponent {

	/**
	 * the users hash
	 */
	public $hash = null;

	/**
	 * copy of the basket
	 */
	public $basket = null;

	/**
	 * setup the cookie component
	 *
	 */
	public function initialize(Controller $controller) {
		parent::initialize($controller);

		if (empty($this->_controller->BasketCookie)) {
			$this->_controller->BasketCookie = $this->_controller->loadComponent('Cookie');
			$this->_controller->BasketCookie->name = 'EvBasket';
			$this->_controller->BasketCookie->key = 'wX$.NC7-2bf2E~K($"ng8=CyETJY^A~.Nx6sEbwA';
			$this->_controller->BasketCookie->time = WEEK; // CakePHP constant - basics.php - sets timestamp for 1 week
		}
	}

	/**
	 * setup a hash or retrieve a hash from the cookie
	 *
	 * @return string  	Basket hash identifier
	 */
	public function setupHash() {
		if (! empty($this->hash)) {
			return $this->hash;
		}

		if ($this->_controller->BasketCookie->check('basketHash')) {
			return $this->hash = $this->_controller->BasketCookie->read('basketHash');
		}

		$this->hash = $this->generateHash();
		$this->_controller->BasketCookie->write(
			'basketHash',
			$this->hash
		);

		return $this->hash;
	}

	/**
	 * given the hash, setup or return the existing basket
	 *
	 * @param 	string 	$hash 	Basket Hash
	 * @return 	bool 	$result True if a basket was created, false if one already exists
	 */
	public function setupBasket($hash) {
		$Basket = EvClassRegistry::init('EvBasket.Basket');

		if (! $Basket->checkExists($hash)) {
			return $Basket->createBasket(
				$hash
			);
		}

		return false;
	}

	/**
	 * build the basket item contain list
	 *
	 * @param 	array  	$itemContain 	Value from Configure::read('EvBasket.BasketItemContains')
	 * @param 	array  	$relationships 	Value from Configure::read('EvBasket.BasketItemBelongsTo')
	 * @return 	array
	 */
	public function buildItemContain($itemContain, $relationships) {
		if (empty($itemContain)) {
			$itemContain = array();

			foreach ($relationships as $key => $value) {
				if (is_array($value)) {
					$itemContain[] = $key;
				} else {
					$itemContain[] = $value;
				}
			}
		}

		return $itemContain;
	}

	/**
	 * @codeCoverageIgnore
	 * generate a hash for the basket
	 * extracted into another method for easier unit testing
	 *
	 * @return string
	 */
	public function generateHash() {
		return CakeText::uuid();
	}

	/**
	 * reset a basket - clear the hash and basket
	 *
	 * @return 	bool
	 */
	public function resetBasket() {
		$this->hash = null;
		$this->basket = null;

		return true;
	}

	/**
	 * get the basket
	 *
	 * @param 	bool 	$rebuild	Whether to force a rebuild of the basket or not
	 * @return 	array 	$basket 	Basket Data
	 */
	public function getBasket($rebuild = false) {
		$this->setupHash();
		$this->setupBasket($this->hash);

		if ($rebuild === false && ! empty($this->basket)) {
			return $this->basket;
		}

		$itemContain = $this->buildItemContain(
			Configure::read('EvBasket.BasketItemContains'),
			Configure::read('EvBasket.BasketItemBelongsTo')
		);

		$Basket = EvClassRegistry::init('EvBasket.Basket');
		$this->basket = $Basket->getFullBasket(
			$this->hash,
			$itemContain
		);

		return $this->basket;
	}

	/**
	 * add an item to the basket
	 *
	 * @param 	array 		$item 		Array with two elements of model and model_id
	 * @param 	decimal 	$unitPrice 	Unit price of the item
	 * @param 	int 		$quantity	Amount of items to add
	 * @return
	 */
	public function addItem($item, $unitPrice, $quantity = 1) {
		$this->setupHash();
	}
}
